@@ -500,9 +500,10 @@ impl<'a> Builder<'a> {
         if mode != Mode::Tool {
             // Tools don't get debuginfo right now, e.g. cargo and rls don't
             // get compiled with debuginfo.
-            cargo.env("RUSTC_DEBUGINFO", self.config.rust_debuginfo.to_string())
-                 .env("RUSTC_DEBUGINFO_LINES", self.config.rust_debuginfo_lines.to_string())
-                 .env("RUSTC_FORCE_UNSTABLE", "1");
+            // Adding debuginfo increases their sizes by a factor of 3-4.
+            cargo.env("RUSTC_DEBUGINFO", self.config.rust_debuginfo.to_string());
+            cargo.env("RUSTC_DEBUGINFO_LINES", self.config.rust_debuginfo_lines.to_string());
+            cargo.env("RUSTC_FORCE_UNSTABLE", "1");
 
             // Currently the compiler depends on crates from crates.io, and
             // then other crates can depend on the compiler (e.g. proc-macro
@@ -625,9 +626,7 @@ impl<'a> Builder<'a> {
                 cargo.arg("--release");
             }
 
-            if mode != Mode::Libstd && // FIXME(#45320)
-               mode != Mode::Libtest && // FIXME(#45511)
-               self.config.rust_codegen_units.is_none() &&
+            if self.config.rust_codegen_units.is_none() &&
                self.build.is_rust_llvm(compiler.host)
             {
                 cargo.env("RUSTC_THINLTO", "1");
