@@ -10,7 +10,6 @@
 
 use std::cmp;
 
-use strings::string_buffer::StringBuffer;
 use syntax::{ast, visit};
 use syntax::attr::HasAttrs;
 use syntax::codemap::{self, BytePos, CodeMap, Pos, Span};
@@ -23,15 +22,15 @@ use comment::{combine_strs_with_missing_comments, contains_comment, remove_trail
               CodeCharKind, CommentCodeSlices, FindUncommented};
 use comment::rewrite_comment;
 use config::{BraceStyle, Config};
-use items::{format_impl, format_trait, rewrite_associated_impl_type, rewrite_associated_type,
-            rewrite_type_alias, FnSig, StaticParts, StructParts};
+use items::{format_impl, format_trait, format_trait_alias, rewrite_associated_impl_type,
+            rewrite_associated_type, rewrite_type_alias, FnSig, StaticParts, StructParts};
 use lists::{itemize_list, write_list, DefinitiveListTactic, ListFormatting, SeparatorPlace,
             SeparatorTactic};
 use macros::{rewrite_macro, MacroPosition};
 use regex::Regex;
 use rewrite::{Rewrite, RewriteContext};
 use shape::{Indent, Shape};
-use utils::{self, contains_skip, inner_attributes, mk_sp, ptr_vec_to_ref_vec};
+use utils::{self, contains_skip, count_newlines, inner_attributes, mk_sp, ptr_vec_to_ref_vec};
 
 fn is_use_item(item: &ast::Item) -> bool {
     match item.node {
@@ -47,18 +46,45 @@ fn is_extern_crate(item: &ast::Item) -> bool {
     }
 }
 
+/// Creates a string slice corresponding to the specified span.
+pub struct SnippetProvider<'a> {
+    /// A pointer to the content of the file we are formatting.
+    big_snippet: &'a str,
+    /// A position of the start of `big_snippet`, used as an offset.
+    start_pos: usize,
+}
+
+impl<'a> SnippetProvider<'a> {
+    pub fn span_to_snippet(&self, span: Span) -> Option<&str> {
+        let start_index = span.lo().to_usize().checked_sub(self.start_pos)?;
+        let end_index = span.hi().to_usize().checked_sub(self.start_pos)?;
+        Some(&self.big_snippet[start_index..end_index])
+    }
+
+    pub fn new(start_pos: BytePos, big_snippet: &'a str) -> Self {
+        let start_pos = start_pos.to_usize();
+        SnippetProvider {
+            big_snippet,
+            start_pos,
+        }
+    }
+}
+
 pub struct FmtVisitor<'a> {
     pub parse_session: &'a ParseSess,
     pub codemap: &'a CodeMap,
-    pub buffer: StringBuffer,
+    pub buffer: String,
     pub last_pos: BytePos,
     // FIXME: use an RAII util or closure for indenting
     pub block_indent: Indent,
     pub config: &'a Config,
     pub is_if_else_block: bool,
+    pub snippet_provider: &'a SnippetProvider<'a>,
+    pub line_number: usize,
+    pub skipped_range: Vec<(usize, usize)>,
 }
 
-impl<'a> FmtVisitor<'a> {
+impl<'b, 'a: 'b> FmtVisitor<'a> {
     pub fn shape(&self) -> Shape {
         Shape::indented(self.block_indent, self.config)
     }
@@ -75,13 +101,17 @@ impl<'a> FmtVisitor<'a> {
                 self.visit_item(item);
             }
             ast::StmtKind::Local(..) | ast::StmtKind::Expr(..) | ast::StmtKind::Semi(..) => {
-                let rewrite = stmt.rewrite(&self.get_context(), self.shape());
-                self.push_rewrite(stmt.span(), rewrite)
+                if contains_skip(get_attrs_from_stmt(stmt)) {
+                    self.push_skipped_with_span(stmt.span());
+                } else {
+                    let rewrite = stmt.rewrite(&self.get_context(), self.shape());
+                    self.push_rewrite(stmt.span(), rewrite)
+                }
             }
             ast::StmtKind::Mac(ref mac) => {
                 let (ref mac, _macro_style, ref attrs) = **mac;
                 if self.visit_attrs(attrs, ast::AttrStyle::Outer) {
-                    self.push_rewrite(stmt.span(), None);
+                    self.push_skipped_with_span(stmt.span());
                 } else {
                     self.visit_mac(mac, None, MacroPosition::Statement);
                 }
@@ -90,7 +120,12 @@ impl<'a> FmtVisitor<'a> {
         }
     }
 
-    pub fn visit_block(&mut self, b: &ast::Block, inner_attrs: Option<&[ast::Attribute]>) {
+    pub fn visit_block(
+        &mut self,
+        b: &ast::Block,
+        inner_attrs: Option<&[ast::Attribute]>,
+        has_braces: bool,
+    ) {
         debug!(
             "visit_block: {:?} {:?}",
             self.codemap.lookup_char_pos(b.span.lo()),
@@ -98,20 +133,16 @@ impl<'a> FmtVisitor<'a> {
         );
 
         // Check if this block has braces.
-        let snippet = self.snippet(b.span);
-        let has_braces = snippet.starts_with('{') || snippet.starts_with("unsafe");
-        let brace_compensation = if has_braces { BytePos(1) } else { BytePos(0) };
+        let brace_compensation = BytePos(if has_braces { 1 } else { 0 });
 
         self.last_pos = self.last_pos + brace_compensation;
         self.block_indent = self.block_indent.block_indent(self.config);
-        self.buffer.push_str("{");
+        self.push_str("{");
 
         if self.config.remove_blank_lines_at_start_or_end_of_block() {
             if let Some(first_stmt) = b.stmts.first() {
                 let attr_lo = inner_attrs
-                    .and_then(|attrs| {
-                        inner_attributes(attrs).first().map(|attr| attr.span.lo())
-                    })
+                    .and_then(|attrs| inner_attributes(attrs).first().map(|attr| attr.span.lo()))
                     .or_else(|| {
                         // Attributes for an item in a statement position
                         // do not belong to the statement. (rust-lang/rust#34459)
@@ -135,7 +166,7 @@ impl<'a> FmtVisitor<'a> {
                     self.last_pos,
                     attr_lo.unwrap_or(first_stmt.span.lo()),
                 ));
-                let len = CommentCodeSlices::new(&snippet)
+                let len = CommentCodeSlices::new(snippet)
                     .nth(0)
                     .and_then(|(kind, _, s)| {
                         if kind == CodeCharKind::Normal {
@@ -169,7 +200,7 @@ impl<'a> FmtVisitor<'a> {
         if !b.stmts.is_empty() {
             if let Some(expr) = utils::stmt_expr(&b.stmts[b.stmts.len() - 1]) {
                 if utils::semicolon_for_expr(&self.get_context(), expr) {
-                    self.buffer.push_str(";");
+                    self.push_str(";");
                 }
             }
         }
@@ -181,7 +212,7 @@ impl<'a> FmtVisitor<'a> {
                     stmt.span.hi(),
                     source!(self, b.span).hi() - brace_compensation,
                 ));
-                let len = CommentCodeSlices::new(&snippet)
+                let len = CommentCodeSlices::new(snippet)
                     .last()
                     .and_then(|(kind, _, s)| {
                         if kind == CodeCharKind::Normal && s.trim().is_empty() {
@@ -220,7 +251,7 @@ impl<'a> FmtVisitor<'a> {
     // The closing brace itself, however, should be indented at a shallower
     // level.
     fn close_block(&mut self, unindent_comment: bool) {
-        let total_len = self.buffer.len;
+        let total_len = self.buffer.len();
         let chars_too_many = if unindent_comment {
             0
         } else if self.config.hard_tabs() {
@@ -229,7 +260,7 @@ impl<'a> FmtVisitor<'a> {
             self.config.tab_spaces()
         };
         self.buffer.truncate(total_len - chars_too_many);
-        self.buffer.push_str("}");
+        self.push_str("}");
         self.block_indent = self.block_indent.block_unindent(self.config);
     }
 
@@ -262,7 +293,7 @@ impl<'a> FmtVisitor<'a> {
 
         if let Some(fn_str) = rewrite {
             self.format_missing_with_indent(source!(self, s).lo());
-            self.buffer.push_str(&fn_str);
+            self.push_str(&fn_str);
             if let Some(c) = fn_str.chars().last() {
                 if c == '}' {
                     self.last_pos = source!(self, block.span).hi();
@@ -274,7 +305,7 @@ impl<'a> FmtVisitor<'a> {
         }
 
         self.last_pos = source!(self, block.span).lo();
-        self.visit_block(block, inner_attrs)
+        self.visit_block(block, inner_attrs, true)
     }
 
     pub fn visit_item(&mut self, item: &ast::Item) {
@@ -294,7 +325,7 @@ impl<'a> FmtVisitor<'a> {
                     // Module is inline, in this case we treat modules like any
                     // other item.
                     if self.visit_attrs(&item.attrs, ast::AttrStyle::Outer) {
-                        self.push_rewrite(item.span, None);
+                        self.push_skipped_with_span(item.span());
                         return;
                     }
                 } else if contains_skip(&item.attrs) {
@@ -320,14 +351,16 @@ impl<'a> FmtVisitor<'a> {
                     attrs = &filtered_attrs;
                 }
             }
-            _ => if self.visit_attrs(&item.attrs, ast::AttrStyle::Outer) {
-                self.push_rewrite(item.span, None);
-                return;
-            },
+            _ => {
+                if self.visit_attrs(&item.attrs, ast::AttrStyle::Outer) {
+                    self.push_skipped_with_span(item.span());
+                    return;
+                }
+            }
         }
 
         match item.node {
-            ast::ItemKind::Use(ref vp) => self.format_import(item, vp),
+            ast::ItemKind::Use(ref tree) => self.format_import(item, tree),
             ast::ItemKind::Impl(..) => {
                 let snippet = self.snippet(item.span);
                 let where_span_end = snippet
@@ -340,6 +373,17 @@ impl<'a> FmtVisitor<'a> {
                 let rw = format_trait(&self.get_context(), item, self.block_indent);
                 self.push_rewrite(item.span, rw);
             }
+            ast::ItemKind::TraitAlias(ref generics, ref ty_param_bounds) => {
+                let shape = Shape::indented(self.block_indent, self.config);
+                let rw = format_trait_alias(
+                    &self.get_context(),
+                    item.ident,
+                    generics,
+                    ty_param_bounds,
+                    shape,
+                );
+                self.push_rewrite(item.span, rw);
+            }
             ast::ItemKind::ExternCrate(_) => {
                 let rw = rewrite_extern_crate(&self.get_context(), item);
                 self.push_rewrite(item.span, rw);
@@ -392,12 +436,12 @@ impl<'a> FmtVisitor<'a> {
                 self.push_rewrite(item.span, rewrite);
             }
             ast::ItemKind::GlobalAsm(..) => {
-                let snippet = Some(self.snippet(item.span));
+                let snippet = Some(self.snippet(item.span).to_owned());
                 self.push_rewrite(item.span, snippet);
             }
             ast::ItemKind::MacroDef(..) => {
                 // FIXME(#1539): macros 2.0
-                let mac_snippet = Some(remove_trailing_white_spaces(&self.snippet(item.span)));
+                let mac_snippet = Some(remove_trailing_white_spaces(self.snippet(item.span)));
                 self.push_rewrite(item.span, mac_snippet);
             }
         }
@@ -407,7 +451,7 @@ impl<'a> FmtVisitor<'a> {
         skip_out_of_file_lines_range_visitor!(self, ti.span);
 
         if self.visit_attrs(&ti.attrs, ast::AttrStyle::Outer) {
-            self.push_rewrite(ti.span, None);
+            self.push_skipped_with_span(ti.span());
             return;
         }
 
@@ -449,7 +493,7 @@ impl<'a> FmtVisitor<'a> {
         skip_out_of_file_lines_range_visitor!(self, ii.span);
 
         if self.visit_attrs(&ii.attrs, ast::AttrStyle::Outer) {
-            self.push_rewrite(ii.span, None);
+            self.push_skipped_with_span(ii.span());
             return;
         }
 
@@ -491,41 +535,63 @@ impl<'a> FmtVisitor<'a> {
         self.push_rewrite(mac.span, rewrite);
     }
 
-    pub fn push_rewrite(&mut self, span: Span, rewrite: Option<String>) {
-        self.format_missing_with_indent(source!(self, span).lo());
-        let result = rewrite.unwrap_or_else(|| self.snippet(span));
-        self.buffer.push_str(&result);
+    pub fn push_str(&mut self, s: &str) {
+        self.line_number += count_newlines(s);
+        self.buffer.push_str(s);
+    }
+
+    fn push_rewrite_inner(&mut self, span: Span, rewrite: Option<String>) {
+        if let Some(ref s) = rewrite {
+            self.push_str(s);
+        } else {
+            let snippet = self.snippet(span);
+            self.push_str(snippet);
+        }
         self.last_pos = source!(self, span).hi();
     }
 
-    pub fn from_codemap(parse_session: &'a ParseSess, config: &'a Config) -> FmtVisitor<'a> {
+    pub fn push_rewrite(&mut self, span: Span, rewrite: Option<String>) {
+        self.format_missing_with_indent(source!(self, span).lo());
+        self.push_rewrite_inner(span, rewrite);
+    }
+
+    pub fn push_skipped_with_span(&mut self, span: Span) {
+        self.format_missing_with_indent(source!(self, span).lo());
+        let lo = self.line_number + 1;
+        self.push_rewrite_inner(span, None);
+        let hi = self.line_number + 1;
+        self.skipped_range.push((lo, hi));
+    }
+
+    pub fn from_context(ctx: &'a RewriteContext) -> FmtVisitor<'a> {
+        FmtVisitor::from_codemap(ctx.parse_session, ctx.config, ctx.snippet_provider)
+    }
+
+    pub fn from_codemap(
+        parse_session: &'a ParseSess,
+        config: &'a Config,
+        snippet_provider: &'a SnippetProvider,
+    ) -> FmtVisitor<'a> {
         FmtVisitor {
             parse_session: parse_session,
             codemap: parse_session.codemap(),
-            buffer: StringBuffer::new(),
+            buffer: String::with_capacity(snippet_provider.big_snippet.len() * 2),
             last_pos: BytePos(0),
             block_indent: Indent::empty(),
             config: config,
             is_if_else_block: false,
+            snippet_provider: snippet_provider,
+            line_number: 0,
+            skipped_range: vec![],
         }
     }
 
-    pub fn opt_snippet(&self, span: Span) -> Option<String> {
-        self.codemap.span_to_snippet(span).ok()
+    pub fn opt_snippet(&'b self, span: Span) -> Option<&'a str> {
+        self.snippet_provider.span_to_snippet(span)
     }
 
-    pub fn snippet(&self, span: Span) -> String {
-        match self.codemap.span_to_snippet(span) {
-            Ok(s) => s,
-            Err(_) => {
-                eprintln!(
-                    "Couldn't make snippet for span {:?}->{:?}",
-                    self.codemap.lookup_char_pos(span.lo()),
-                    self.codemap.lookup_char_pos(span.hi())
-                );
-                "".to_owned()
-            }
-        }
+    pub fn snippet(&'b self, span: Span) -> &'a str {
+        self.opt_snippet(span).unwrap()
     }
 
     // Returns true if we should skip the following item.
@@ -629,9 +695,8 @@ impl<'a> FmtVisitor<'a> {
         // Extract leading `use ...;`.
         let items: Vec<_> = stmts
             .iter()
-            .take_while(|stmt| to_stmt_item(stmt).is_some())
+            .take_while(|stmt| to_stmt_item(stmt).map_or(false, is_use_item))
             .filter_map(|stmt| to_stmt_item(stmt))
-            .take_while(|item| is_use_item(item))
             .collect();
 
         if items.is_empty() {
@@ -661,15 +726,17 @@ impl<'a> FmtVisitor<'a> {
         let is_internal = !(inner_span.lo().0 == 0 && inner_span.hi().0 == 0)
             && local_file_name == self.codemap.span_to_filename(inner_span);
 
-        self.buffer.push_str(&*utils::format_visibility(vis));
-        self.buffer.push_str("mod ");
-        self.buffer.push_str(&ident.to_string());
+        self.push_str(&*utils::format_visibility(vis));
+        self.push_str("mod ");
+        self.push_str(&ident.to_string());
 
         if is_internal {
             match self.config.brace_style() {
-                BraceStyle::AlwaysNextLine => self.buffer
-                    .push_str(&format!("\n{}{{", self.block_indent.to_string(self.config))),
-                _ => self.buffer.push_str(" {"),
+                BraceStyle::AlwaysNextLine => {
+                    let sep_str = format!("\n{}{{", self.block_indent.to_string(self.config));
+                    self.push_str(&sep_str);
+                }
+                _ => self.push_str(" {"),
             }
             // Hackery to account for the closing }.
             let mod_lo = self.codemap.span_after(source!(self, s), "{");
@@ -677,7 +744,7 @@ impl<'a> FmtVisitor<'a> {
                 self.snippet(mk_sp(mod_lo, source!(self, m.inner).hi() - BytePos(1)));
             let body_snippet = body_snippet.trim();
             if body_snippet.is_empty() {
-                self.buffer.push_str("}");
+                self.push_str("}");
             } else {
                 self.last_pos = mod_lo;
                 self.block_indent = self.block_indent.block_indent(self.config);
@@ -688,7 +755,7 @@ impl<'a> FmtVisitor<'a> {
             }
             self.last_pos = source!(self, m.inner).hi();
         } else {
-            self.buffer.push_str(";");
+            self.push_str(";");
             self.last_pos = source!(self, s).hi();
         }
     }
@@ -722,6 +789,7 @@ impl<'a> FmtVisitor<'a> {
             use_block: false,
             is_if_else_block: false,
             force_one_line_chain: false,
+            snippet_provider: self.snippet_provider,
         }
     }
 }
@@ -796,10 +864,10 @@ impl Rewrite for ast::Attribute {
                     .unwrap_or(0),
                 ..shape
             };
-            rewrite_comment(&snippet, false, doc_shape, context.config)
+            rewrite_comment(snippet, false, doc_shape, context.config)
         } else {
-            if contains_comment(&snippet) {
-                return Some(snippet);
+            if contains_comment(snippet) {
+                return Some(snippet.to_owned());
             }
             // 1 = `[`
             let shape = shape.offset_left(prefix.len() + 1)?;
@@ -830,7 +898,7 @@ where
             // Extract comments between two attributes.
             let span_between_attr = mk_sp(attr.span.hi(), next_attr.span.lo());
             let snippet = context.snippet(span_between_attr);
-            if snippet.chars().filter(|c| *c == '\n').count() >= 2 || snippet.contains('/') {
+            if count_newlines(snippet) >= 2 || snippet.contains('/') {
                 break;
             }
         }
@@ -872,10 +940,7 @@ fn rewrite_first_group_attrs(
             for derive in derives {
                 derive_args.append(&mut get_derive_args(context, derive)?);
             }
-            return Some((
-                derives.len(),
-                format_derive(context, &derive_args, shape)?,
-            ));
+            return Some((derives.len(), format_derive(context, &derive_args, shape)?));
         }
     }
     // Rewrite the first attribute.
@@ -886,7 +951,7 @@ fn has_newlines_before_after_comment(comment: &str) -> (&str, &str) {
     // Look at before and after comment and see if there are any empty lines.
     let comment_begin = comment.chars().position(|c| c == '/');
     let len = comment_begin.unwrap_or_else(|| comment.len());
-    let mlb = comment.chars().take(len).filter(|c| *c == '\n').count() > 1;
+    let mlb = count_newlines(&comment[..len]) > 1;
     let mla = if comment_begin.is_none() {
         mlb
     } else {
@@ -919,7 +984,7 @@ impl<'a> Rewrite for [ast::Attribute] {
             // Preserve an empty line before/after doc comments.
             if self[0].is_sugared_doc || self[first_group_len].is_sugared_doc {
                 let snippet = context.snippet(missing_span);
-                let (mla, mlb) = has_newlines_before_after_comment(&snippet);
+                let (mla, mlb) = has_newlines_before_after_comment(snippet);
                 let comment = ::comment::recover_missing_comment_in_span(
                     missing_span,
                     shape.with_max_width(context.config),
@@ -953,7 +1018,7 @@ impl<'a> Rewrite for [ast::Attribute] {
 }
 
 // Format `#[derive(..)]`, using visual indent & mixed style when we need to go multiline.
-fn format_derive(context: &RewriteContext, derive_args: &[String], shape: Shape) -> Option<String> {
+fn format_derive(context: &RewriteContext, derive_args: &[&str], shape: Shape) -> Option<String> {
     let mut result = String::with_capacity(128);
     result.push_str("#[derive(");
     // 11 = `#[derive()]`
@@ -995,7 +1060,7 @@ fn is_derive(attr: &ast::Attribute) -> bool {
 }
 
 /// Returns the arguments of `#[derive(...)]`.
-fn get_derive_args(context: &RewriteContext, attr: &ast::Attribute) -> Option<Vec<String>> {
+fn get_derive_args<'a>(context: &'a RewriteContext, attr: &ast::Attribute) -> Option<Vec<&'a str>> {
     attr.meta().and_then(|meta_item| match meta_item.node {
         ast::MetaItemKind::List(ref args) if meta_item.name.as_str() == "derive" => {
             // Every argument of `derive` should be `NestedMetaItemKind::Literal`.
@@ -1013,10 +1078,19 @@ fn get_derive_args(context: &RewriteContext, attr: &ast::Attribute) -> Option<Ve
 pub fn rewrite_extern_crate(context: &RewriteContext, item: &ast::Item) -> Option<String> {
     assert!(is_extern_crate(item));
     let new_str = context.snippet(item.span);
-    Some(if contains_comment(&new_str) {
-        new_str
+    Some(if contains_comment(new_str) {
+        new_str.to_owned()
     } else {
         let no_whitespace = &new_str.split_whitespace().collect::<Vec<&str>>().join(" ");
         String::from(&*Regex::new(r"\s;").unwrap().replace(no_whitespace, ";"))
     })
 }
+
+fn get_attrs_from_stmt(stmt: &ast::Stmt) -> &[ast::Attribute] {
+    match stmt.node {
+        ast::StmtKind::Local(ref local) => &local.attrs,
+        ast::StmtKind::Item(ref item) => &item.attrs,
+        ast::StmtKind::Expr(ref expr) | ast::StmtKind::Semi(ref expr) => &expr.attrs,
+        ast::StmtKind::Mac(ref mac) => &mac.2,
+    }
+}
